// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *
 * 			Validating server-side TCP streams
 *
 * Simple proxying server
 * This code listens on a specified port. When a new connection
 * comes, the server accepts it, and launches the specified program
 *
 * $Id: simple-proxy.cc,v 1.1 2005/02/18 08:00:57 oleg Exp oleg $
 *
 ************************************************************************
 */

#include "TCPstream.h"
#include "myenv.h"
#if defined(__GNUC__) && __GNUC__ >= 3
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#else
#include <iostream.h>
#endif
#include <string.h>
#include <stdlib.h>
#include <time.h>
#if defined(unix)
#include <unistd.h>
#include <sys/wait.h>
#endif

static char Command [] = "/bin/cat";

				// Configurational parameters: adjust
				// if necessary
static int Server_port = 3926;

				// Handle a client
static void handle_client(int link)
{
  				// Fork from a parent, if desired
  const pid_t kid_id = fork();
  if( kid_id != 0 )
  {			// We're in the parent process
    if( kid_id < 0 )
      CurrentNetCallback::on_error("Fork error");
    cerr << "\nLaunched a process " << kid_id << " to handle a new client"
	 << endl;
    return;			// The kid will do all the work...
  }

  dup2(link,0);		/* re-direct both stdin and stdout */
  dup2(link,1);
  execl("/bin/sh", "sh", "-c", Command, 0);
  _exit(127);		// only if execl failed
}

				// Root module
int main(void)
{
  cerr << "\n\nValidating TCPstream module...\n" << endl;
  cerr << "\nStaring a "
       << "forking "
       << "server on port " << Server_port << endl;

  SocketAddr server_addr = SocketAddr(IPaddress(),Server_port);
  
  int listening_socket = socket(AF_INET,SOCK_STREAM,0);
  if( listening_socket < 0 )
   CurrentNetCallback::on_error("Socket creation error");
       
  {
    int value = 1;
    if( setsockopt(listening_socket, SOL_SOCKET, SO_REUSEADDR,
		   (char*)&value, sizeof(value)) < 0 )
      CurrentNetCallback::on_error("setsockopt SO_REUSEADDR error");
  }

  if( bind(listening_socket, (sockaddr *)server_addr,sizeof(server_addr)) < 0 )
   CurrentNetCallback::on_error("Socket binding error");

  if( listen(listening_socket, 5) < 0 )
   CurrentNetCallback::on_error("Socket listen error");
  
  cerr << "Entering the server loop..." << endl;
  
  TCPstream link;
  for(;;)
  {
    struct sockaddr peeraddr;
    socklen_t peeraddr_len = 0;
    int link = accept(listening_socket,&peeraddr,&peeraddr_len);
    if (link < 0) 
      CurrentNetCallback::on_error("Socket accept error");
    cerr << "\nAccepted connection from " << endl;
    handle_client(link);
    close(link);
    ::wait4(-1,0,WNOHANG,0);  // collect zombies, if any
   }
  while( ::wait(0) > 0 );	// wait until all forked children finish...
  close(listening_socket);
  cerr << "\nAll Done\n" << endl;
  return 0;
}
